var renderClass = "jp.ngt.rtm.render.VehiclePartsRenderer";
importPackage(Packages.org.lwjgl.opengl);
importPackage(Packages.jp.ngt.rtm.render);
importPackage(Packages.jp.ngt.ngtlib.math);
importPackage(Packages.jp.ngt.rtm)
importPackage(Packages.jp.ngt.rtm.entity.train.util);
importPackage(Packages.jp.ngt.ngtlib.io);
importPackage(Packages.jp.ngt.rtm.entity.vehicle);
importPackage(Packages.jp.ngt.ngtlib.util);
importPackage(Packages.jp.ngt.ngtlib.renderer);
//データ保存処理(グローバル関数)
var LocalData = new java.util.HashMap();


//##### 初期化 ####################
function init(par1, par2) {
	//車両設定////////////////////////////////////////

	bogieSet = {};
	bogieSet.bd0 = 5.0; //台車間距離(中間台車は車体中央、前後台車は対称位置とする)
	bogieSet.wd0 = 1.15; //軸距÷2(前後台車)
	bogieSet.wd1 = 1.15; //軸距÷2(中間台車)
	bogieSet.wy0 = -0.527; //車輪軸Y座標(前後台車)
	bogieSet.wy1 = -0.527; //車輪軸Y座標(中間台車)

	pantaDistance = 5.65; //パンタ中心のZ位置（TTK+20m:7.0／18m：6.0／17&16m：5.5／10m：3.0）
	pantaType = "Normal"; //パンタ高(Normal:標準-格納 / W51:W51-格納 / Compatible:標準-W51)


	//オブジェクト定義////////////////////////////////////////

	//車体
	body = renderer.registerParts(new Parts("body", "body-s", "conectionF", "window", "conectionB", "parts", "cooler", "under", "camera")); //車体
	lightFon = renderer.registerParts(new Parts("light-Fon"));
	lightFof = renderer.registerParts(new Parts("light-Fof"));
	lightBon = renderer.registerParts(new Parts("light-Bon"));
	lightBof = renderer.registerParts(new Parts("light-Bof"));

	//台車
	bogieF = renderer.registerParts(new Parts("F-TR202A")); //前台車
	bogieM = renderer.registerParts(new Parts("M-TR202A")); //中台車
	bogieB = renderer.registerParts(new Parts("B-TR202A")); //後台車
	wheelF1 = renderer.registerParts(new Parts("wheelF1")); //車輪
	wheelF2 = renderer.registerParts(new Parts("wheelF2"));
	wheelM1 = renderer.registerParts(new Parts("wheelM1"));
	wheelM2 = renderer.registerParts(new Parts("wheelM2"));
	wheelB1 = renderer.registerParts(new Parts("wheelB1"));
	wheelB2 = renderer.registerParts(new Parts("wheelB2"));


	//三角関数配列////////////////////////////////////////
	sinArray = [];
	for (var i = 0; i < 6284; i++) {
		sinArray[i] = Math.sin(i / 1000);
	}
	cosArray = [];
	for (var i = 0; i < 6284; i++) {
		cosArray[i] = Math.cos(i / 1000);
	}
}

//##### 三角関数 ####################
function getTri(par1, par2) {
	//getSin系が重いので配列化して参照
	//par1=sinでrad->sin、par1=Dsinでdeg->sin

	//Deg->Rad
	if (par1.match(/[Dd]/)) {
		par2 = par2 * Math.PI / 180;
	}

	//丸め
	var x = par2 % 6.284 * 1000;
	x = Math.round(x);
	x = Math.abs(x);

	var isSin = Boolean(par1.match("sin"));
	var isCos = Boolean(par1.match("cos"));

	//正負
	var isN = 1.0;
	if (par2 < 0) {
		isN = -1.0;
	}

	switch (true) {
		case isSin:
			var y = sinArray[x] * isN;
			break;
		case isCos:
			var y = cosArray[x];
			break;
		default:
			break;
	}

	return y;
}

//##### 両数取得 ####################
function isCarSize(entity) {
    var formation = entity.getFormation();//formation型
    if (formation == null) {//formationが取得できないときの処理
        return 1;
    }
    var max = formation.size();//連結してる両数を返す
    return max;
}

function MCVersionChecker() {
	var varsion = RTMCore.VERSION;
	if (varsion.indexOf("1.7.10") >= 0) return "1.7.10";
	else if (varsion.indexOf("2.0") >= 0) return "1.8.9";
	else if (varsion.indexOf("2.1") >= 0) return "1.9.4";
	else if (varsion.indexOf("2.2") >= 0) return "1.10.2";
	else if (varsion.indexOf("2.4") >= 0) return "1.12.2";
	else return "unknown";
}

//##### 号車取得 ####################
function isMiddleCar(entity) {//trueなら中間車、falseなら先頭車/後尾車

	//-通さない処理-
	if (entity === null) return;
	//if (renderer.currentMatId !== 2) return;//材質でも走査されるため

	var data = LocalData.get(entity);//保存された"データ群"の呼び出し
	var state = false;

	var formation = entity.getFormation();//formation型
	if (formation == null) {//formationが取得できないときの処理
		state = data.prevstate;
		if (data.prevstate == null) state = false;
	} else {
		var current = formation.getEntry(entity).entryId + 1;//この車両の号車を取得。0から数えるため+1する
		var max = formation.size();//連結してる両数を返す
		if (current == 1 || current == max) {//この車両が先頭or最後尾なら
			state = false;
		} else {//そうでない(つまり中間)なら
			state = true;
		}
	}

	data.prevstate = state;
	LocalData.put(entity, data);

	return state;
}

//##### render ####################
function render(entity, pass, par3) {	//データ保存処理の初期化
	//Entityをキーにしてるので競合を起こさない(setData/getDataのビットシフトより確実性が高い)
	//必ず一番最初に実行されるように書く
	var data = LocalData.get(entity) || {};
	LocalData.put(entity, data);

	var Middle = isMiddleCar(entity);
	var varsion = MCVersionChecker();


	GL11.glPushMatrix();

	//通常・半透明描画
	if (pass == 0 || pass == 1) {
		body.render(renderer);
		render_bogie(entity);
		render_light(entity, Middle, pass);

	}

	//ライト描画
	if (pass > 1) {
		render_light(entity, Middle, pass);
	}

	GL11.glPopMatrix();
}

//##### 隣接車両取得 ####################
function isFrontNextCar(entity) {//trueなら車両前方に、falseなら車両後方に車両あり※先頭車でのみ実行
    var frontNext = entity.getConnectedTrain(0);
    if (frontNext == null) {//前方に隣接車の情報が取得できない≒前方に隣接車なし＝後方に隣接車あり
        return false;
    } else {
        return true;
    }
}

//##### render_台車 ####################
function render_bogie(entity) {

	//初期化
	var f0 = 0.0,
		bodyYaw = 0.0,
		bogieFYaw = 0.0,
		bogieBYaw = 0.0,
		bogieMYaw = 0.0,
		bodyPitch = 0.0,
		bogieFPitch = 0.0,
		bogieBPitch = 0.0,
		slideX = 0.0,
		slideY = 0.0;


	//取得
	if (entity != null && entity.getBogie(0) != null && entity.getBogie(1) != null) {
		f0 = renderer.getWheelRotationR(entity);

		bodyYaw = entity.field_70177_z;

		//前台車ヨーローカル角
		var i = entity.getBogie(0).field_70177_z - bodyYaw;
		if (i > 180) {
			bogieFYaw = i - 360;
		} else if (i < -180) {
			bogieFYaw = i + 360;
		} else {
			bogieFYaw = i;
		}
		//後台車ヨーローカル角
		var i = entity.getBogie(1).field_70177_z - bodyYaw - 180;
		if (i > 180) {
			bogieBYaw = i - 360;
		} else if (i < -180) {
			bogieBYaw = i + 360;
		} else {
			bogieBYaw = i;
		}

		//ピッチローカル角
		bodyPitch = entity.field_70125_A;
		bogieFPitch = entity.getBogie(0).field_70125_A * -1 + bodyPitch;
		bogieBPitch = entity.getBogie(1).field_70125_A + bodyPitch;

	}


	//計算
	//中台車横偏位
	var FYawR = bogieSet["bd0"] / getTri("Dsin", bogieFYaw);
	if (isFinite(FYawR)) {
		var slideXF = FYawR - FYawR * getTri("Dcos", bogieFYaw);
	} else {
		var slideXF = 0.0;
	}
	var BYawR = bogieSet["bd0"] / getTri("Dsin", bogieBYaw);
	if (isFinite(BYawR)) {
		var slideXB = BYawR - BYawR * getTri("Dcos", bogieBYaw);
	} else {
		var slideXB = 0.0;
	}
	slideX = (slideXB - slideXF) / 2;

	//中台車縦偏位
	var FPitchR = bogieSet["bd0"] / getTri("Dsin", bogieFPitch);
	if (isFinite(FPitchR)) {
		var slideYF = FPitchR - FPitchR * getTri("Dcos", bogieFPitch);
	} else {
		var slideYF = 0.0;
	}
	var BPitchR = bogieSet["bd0"] / getTri("Dsin", bogieBPitch);
	if (isFinite(BPitchR)) {
		var slideYB = BPitchR - BPitchR * getTri("Dcos", bogieBPitch);
	} else {
		var slideYB = 0.0;
	}
	slideY = (slideYF - slideYB) / 2;

	//中台車ヨー回転
	bogieMYaw = (bogieFYaw + bogieBYaw) * -0.5;


	//表示
	//前台車
	GL11.glPushMatrix();
	renderer.rotate(bogieFYaw, 'Y', 0.0, 0.0, bogieSet["bd0"]);
	renderer.rotate(bogieFPitch, 'X', 0.0, 0.0, bogieSet["bd0"]);
	bogieF.render(renderer);
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], bogieSet["bd0"] + bogieSet["wd0"]);
	wheelF1.render(renderer);
	GL11.glPopMatrix();
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], bogieSet["bd0"] - bogieSet["wd0"]);
	wheelF2.render(renderer);
	GL11.glPopMatrix();
	GL11.glPopMatrix();

	//中台車
	GL11.glPushMatrix();
	GL11.glTranslatef(slideX, slideY, 0.0);
	renderer.rotate(bogieMYaw, 'Y', 0.0, 0.0, 0.0);
	bogieM.render(renderer);
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy1"], bogieSet["wd1"]);
	wheelM1.render(renderer);
	GL11.glPopMatrix();
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy1"], -bogieSet["wd1"]);
	wheelM2.render(renderer);
	GL11.glPopMatrix();
	GL11.glPopMatrix();

	//後台車
	GL11.glPushMatrix();
	renderer.rotate(bogieBYaw, 'Y', 0.0, 0.0, -bogieSet["bd0"]);
	renderer.rotate(bogieBPitch, 'X', 0.0, 0.0, -bogieSet["bd0"]);
	bogieB.render(renderer);
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], -bogieSet["bd0"] + bogieSet["wd0"]);
	wheelB1.render(renderer);
	GL11.glPopMatrix();
	GL11.glPushMatrix();
	renderer.rotate(f0, 'X', 0.0, bogieSet["wy0"], -bogieSet["bd0"] - bogieSet["wd0"]);
	wheelB2.render(renderer);
	GL11.glPopMatrix();
	GL11.glPopMatrix();
}

//##### render_ライト ####################
function render_light(entity, Middle, pass) {

	var varsion = MCVersionChecker();
	var st11 = 0; //方向幕の一番上から普通幕の終わりまでのコマ数

	if (entity != null) {
		var form = isMiddleCar(entity);
		var Fnext = isFrontNextCar(entity);
		var max = isCarSize(entity);
	}



	if (entity != null) {
		if (varsion == "1.7.10" || varsion == "1.8.9" || varsion == "1.9.4") {
			var st0 = entity.getTrainStateData(0); //進行方向
		} else {
			var st0 = entity.getVehicleState(TrainState.getStateType(0)); //進行方向
		}
	}

	if (max > 1) {//単行でないとき
		if (form == false) {//編成両端であるとき
			if (Fnext == false) {//自車後方にほかの車両があるとき
				if (st0 == 0) {//前方に進むとき(=進行方向最前)
					GL11.glPushMatrix();

					lightFon.render(renderer);
					lightBof.render(renderer);


					GL11.glPopMatrix();
				} else {//後方に進むとき(=進行方向最後)
					GL11.glPushMatrix();

					lightFof.render(renderer);
					lightBon.render(renderer);

					GL11.glPopMatrix();
				}
			} else {//自車前方にほかの車両があるとき
				if (st0 == 0) {//前方に進むとき(=進行方向最後)
					GL11.glPushMatrix();

					lightFof.render(renderer);
					lightBof.render(renderer);


					GL11.glPopMatrix();
				} else {//後方に進むとき(=進行方向最前)
					GL11.glPushMatrix();

					lightFof.render(renderer);
					lightBon.render(renderer);


					GL11.glPopMatrix();
				}
			}
		} else {//編成中間であるとき
			GL11.glPushMatrix();

			lightFof.render(renderer);
			lightBof.render(renderer);


			GL11.glPopMatrix();
		}
	} else {//単行であるとき
		if (st0 == 0) {//前方に進むとき
			GL11.glPushMatrix();
			lightFon.render(renderer);
			lightBof.render(renderer);
			GL11.glPopMatrix();
		} else {//後方に進むとき
			GL11.glPushMatrix();
			lightFof.render(renderer);
			lightBon.render(renderer);


			GL11.glPopMatrix();
		}
	}

}


